/*
 * advanced-vanadiel-clock.js
 *
 * Copyright (c) 2005 by Daniel Potter.  All rights reserved.
 *
 * This file is part of the Vana'diel Clock.
 *
 * The Vana'diel Clock is free software; you can redistribute
 * it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * The Vana'diel Clock is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the Vana'diel Clock; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 *
 * A more complicated clock, this handles the clock for the "advanced" display
 * and contains two ticks - Earth time and Vanadiel time.
 */

/*** AIRSHIP CONSTANTS ***/
// These are based on the Mithra Pride airship timer found at
// http://www.mithrapride.org/vana_time/airshiptable.html
//
// I haven't checked these yet myself - the Kazham and Windurst times
// seem to be a little off.

// Amount of time the airship takes, total, between trips (6 Vana'diel hours,
// approximately 15 Earth minutes - note this includes boarding time)
const airshipDelay = 6 * 3600;

// Offsets from "base time" to the various airship departure times.

var airshipTimeDelays = [
    4 * 3600 + 10 * 60, 1 * 3600 + 10 * 60,  // San d'Oria in/out
    1 * 3600 + 10 * 60, 4 * 3600 + 10 * 60,   // Bastok in/out
    5 * 3600 + 45 * 60, 2 * 3600 + 40 * 60,   // Windurst in/out
    2 * 3600 + 40 * 60, 5 * 3600 + 35 * 60 ]; // Kazham in/out

// Cell names to place calculate airship times into
var airshipIds = [
    "sandoria.inbound", "sandoria.outbound",
    "bastok.inbound", "bastok.outbound",
    "windurst.inbound", "windurst.outbound",
    "kazham.inbound", "kazham.outbound"
];

var vanadielOffset;

// Ferry delay (time it takes for the ferry to travel between ports)
var ferryDelay = 28800;

// Object used for representing the races of Vana'diel (used for the RSE
// timer)
function VanaRace(name, icon) {
    this.name = name;
    this.icon = icon;
}

// The various RSE races, in RSE order

var RSERaces = [
    new VanaRace('elvaan.male', 'elvaan-m-icon'),
    new VanaRace('elvaan.female', 'elvaan-f-icon'),
    new VanaRace('tarutaru.male', 'tarutaru-m-icon'),
    new VanaRace('tarutaru.female', 'tarutaru-f-icon'),
    new VanaRace('mithra', 'mithra-icon'),
    new VanaRace('galka', 'galka-icon'),
    new VanaRace('hume.male', 'hume-m-icon'),
    new VanaRace('hume.female', 'hume-f-icon')
];

var RSEActive = "?";

var RSELocations = [
    "rse.ordelle",
    "rse.gusgen",
    "rse.shakhrami",
];

const BALLISTA_ROWS = 9;

function Guild(name, holiday, open, close) {
    this.name = name;
    this.icon = name;
    this.holiday = holiday;
    this.open = open;
    this.close = close;
    this.changeAt = new Date();
}

// Guild start/stop times from Mysterytour

var guilds = [
    new Guild("alchemy", 6, 8, 23),      // Holiday: Lightday
    new Guild("bonecraft", 3, 8, 23),    // Holiday: Windsday
    new Guild("clothcraft", 0, 6, 21),   // Holiday: Fireday
    new Guild("cooking", 7, 5, 20),      // Holiday: Darkday
    new Guild("fishing", 5, 3, 18),      // Holiday: Thunderday
    new Guild("goldsmithing", 4, 8, 23), // Holiday: Iceday
    new Guild("leathercraft", 4, 3, 18), // Holiday: Iceday
    new Guild("smithing", 2, 8, 23),     // Holiday: Watersday
    new Guild("woodworking", 0, 6, 21)   // Holiday: Fireday
];

var guildLabels = new Array(guilds.length);

var guildOpen = "O";
var guildClosed = "-";
var guildHoliday = "H";
var guildOpenAt = "Open:";
var guildCloseAt = "Close:";

var ballistaEntry = "Entry";
var ballistaGuestEntry = "Guest Entry";
var ballistaOpenEntry = "Open Entry";
var ballistaCloseEntry = "Entry Closes";
var ballistaMatch = "Match";

var vanadielClock;

function VanadielClock_advancedLoad(event) {
    // init base strings
    VanadielClock_init();
    // now init our strings
    var strBundleService = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService);
    var strings = strBundleService.createBundle("chrome://vanadielclock/locale/advanced-clock.properties");

    for (var i in RSERaces) {
	try {
	    RSERaces[i].name = strings.GetStringFromName("race." + RSERaces[i].name);
	} catch (ex) {
	    alert(RSERaces[i].name + " not localized!\n\nThis message not localized!");
	}
    }
    for (var i in RSELocations) {
	RSELocations[i] = strings.GetStringFromName(RSELocations[i]);
    }
    for (var i in guilds) {
	guilds[i].name = strings.GetStringFromName("guild." + guilds[i].name);
    }
    RSEActive = strings.GetStringFromName("rse.active");
    guildOpen = strings.GetStringFromName("guild.status.open");
    guildClosed = strings.GetStringFromName("guild.status.closed");
    guildHoliday = strings.GetStringFromName("guild.status.holiday");
    guildOpenAt = strings.GetStringFromName("guild.opensAt");
    guildCloseAt = strings.GetStringFromName("guild.closesAt");
    ballistaEntry = strings.GetStringFromName("ballista.entry.starts");
    ballistaGuestEntry = strings.GetStringFromName("ballista.entry.guest.starts");
    ballistaOpenEntry = strings.GetStringFromName("ballista.entry.open.starts");
    ballistaCloseEntry = strings.GetStringFromName("ballista.entry.closes");
    ballistaMatch = strings.GetStringFromName("ballista.match.starts");

    localizeTimeFormatStrings();
    VanadielClock_localizeBallista();

    // Call build functions to create UI elements
    createRSERows();
    createGuildRows();
    createBallistaRows(strings.GetStringFromName("ballista.versus"),
    			strings.GetStringFromName("ballista.at"),
			strings.GetStringFromName("ballista.entry.starts.long"));

    // Switch to the appropriate tab
    document.getElementById("detail-tabs").selectedIndex = Components.classes["@mozilla.org/preferences-service;1"].
               getService(Components.interfaces.nsIPrefBranch).getIntPref("vanadiel.clock.tab");

    // and create the clock (autostarts)
    vanadielClock = new AdvancedVanadielClock();
}

function VanadielClock_advancedUnload(event) {
    vanadielClock.destroy();
    vanadielClock = null;
}

function AdvancedVanadielClock() {
    this.dayIcon = document.getElementById("day-icon");
    this.dateLabel = document.getElementById("date-label");
    this.timeLabel = document.getElementById("timer-label");
    this.moonIcon = document.getElementById("moon-phase-icon");
    this.moonLabel = document.getElementById("moon-phase-label");
    this.earthTimeLabel = document.getElementById("earth-label");
    this.nextFullMoonLabel = document.getElementById("moon-next-full");
    this.nextNewMoonLabel = document.getElementById("moon-next-new");
    this.nextFerryLabel = document.getElementById("ferry-timer");
    this.nextRSELabel = document.getElementById("rse-time-label-1");
    this.ballistaLabel = document.getElementById("ballisa-text");
    this.ballistaClosesLabel = document.getElementById("ballista-entry-timer");
    this.airshipElements = new Array(airshipIds.length);
    for (var i in airshipIds) {
	this.airshipElements[i] = document.getElementById(airshipIds[i]);
    }
    this.airshipTimes = [ 0, 0, 0, 0, 0, 0, 0, 0 ];
    this.guildTimers = new Array(guilds.length);
    for (var i in guilds) {
	this.guildTimers[i] = document.getElementById("guild-timer-label-" + i);
    }
    // Grab the clock
    this.clock = VanadielClock_TimerService;
    if (!this.clock.active) {
	this.clock.start();
    }
    vanadielOffset = this.clock.vanadielOffset;
    // apparently the following don't get set?
    this.lastDay = 0;
    this.lastHour = -1;
    this.vanadielTicked(this.clock.year, this.clock.month, this.clock.day, this.clock.weekday, this.clock.hour, this.clock.minute);
    this.notify(null);
    this.clock.addVanadielClockListener(this);
    this.resetToBoundary = true;
    this.timer = Components.classes["@mozilla.org/timer;1"].createInstance(Components.interfaces.nsITimer);
    this.timer.initWithCallback(this, 1000 - (new Date().getTime() % 1000), Components.interfaces.nsITimer.TYPE_ONE_SHOT);
}

AdvancedVanadielClock.prototype = {
    clock: null,
    lastDay: 0,
    lastHour: 0,
    moonIcon: null,
    dayIcon: null,
    dateLabel: null,
    timeLabel: null,
    moonIcon: null,
    moonLabel: null,
    earthTimeLabel: null,
    nextFullMoonLabel: null,
    nextNewMoonLabel: null,
    nextFerryLabel: null,
    earthNextFerry: 0,
    timer: null,
    airshipElements: null,
    vanadielNextFullMoon: 0,
    vanadielNextNewMoon: 0,
    nextFullMoonDate: 0,
    nextNewMoonDate: 0,
    nextRSELabel: null,
    nextRSEDate: 0,
    airshipTimes: null,
    guildTimers: null,
    nextBallista: null,
    nextBallistaCloses: null,
    ballistaLabel: null,
    ballistaClosesLabel: null,

    destroy: function() {
	this.timer.cancel();
	this.timer = null;
	this.clock.removeVanadielClockListener(this);
	this.clock = null;
    },

    // Callback for the clock component
    vanadielTicked: function(year, month, day, weekday, hour, minute) {
	// Only update day related stuff on - well, a new day
	var vanadielDay = this.clock.epochDays;
	var vanadielTime = this.clock.time;
	if (vanadielDay != this.lastDay) {
	    this.lastDay = vanadielDay;
	    var date = vanaLocalNames[weekday] + ", " + day + " / ";
	    date += month + " / ";
	    date += year;
	    var basicMoonPhase = this.clock.basicMoonPhase;
	    var moonPhase = this.clock.moonPhase;
	    var moonPhaseLabel = this.clock.moonPercent + "% " + moonPhaseLocalNames[basicMoonPhase];
	    if (basicMoonPhase == 0 || basicMoonPhase == 4) {
		// add "waxing" or "waning" to this if you can't tell
		// (ie, when the moon is new or full, so you know if it's
		// moving to/from the "true" full/new state)
		if (moonPhase != 0 && moonPhase != 42) {
		    // But don't bother showing waxing/waning when the moon
		    // really is completely full or completely new - it should
		    // (hopefully) be obvious that it's going to move in the
		    // other direction.
		    moonPhaseLabel += " (" + (this.clock.waxing ? moonPhaseLocalWaxing : moonPhaseLocalWaning) + ")";
		}
	    }
	    // Calculate the next full moon and the next new moon.
	    this.vanadielNextFullMoon = vanadielDay + (80 - moonPhase);
	    this.vanadielNextNewMoon = vanadielDay + (38 - moonPhase);
	    // If we're currently full, jump to the next one
	    if (this.vanadielNextFullMoon < vanadielDay)
		this.vanadielNextFullMoon += 84;
	    if (this.vanadielNextNewMoon < vanadielDay)
		this.vanadielNextNewMoon += 84;
	    this.vanadielNextFullMoon *= 86400;
	    this.vanadielNextNewMoon *= 86400;
	    this.nextFullMoonDate = VanadielClock_convertToEarthDate(this.vanadielNextFullMoon);
	    this.nextNewMoonDate = VanadielClock_convertToEarthDate(this.vanadielNextNewMoon);
	    this.moonIcon.setAttribute("tooltiptext", moonPhaseLabel);
	    this.moonIcon.setAttribute("phase", moonPhases[basicMoonPhase]);
	    this.moonLabel.setAttribute("value", moonPhaseLabel);
	    this.dayIcon.setAttribute("tooltiptext", date);
	    this.dayIcon.setAttribute("element", vanaDays[weekday]);
	    this.dateLabel.setAttribute("value", date);
	    this.timeLabel.setAttribute("tooltiptext", date);
	    this.updateRSE();
	}
	var label = hour + ":";
	if (minute < 10) {
	    label += "0";
	}
	label += minute;
	this.timeLabel.setAttribute("value", label);
	if (this.lastHour != hour) {
	    this.lastHour = hour;
	    // Calculate the next ferry time
	    this.earthNextFerry = VanadielClock_convertToEarthDate(Math.floor(vanadielTime / ferryDelay) * ferryDelay + ferryDelay);
	    this.updateGuilds(vanadielDay, hour);
	    this.updateBallista();
	    // since this can change the window sizing, "fix it"
	    // I want a more long term solution, but, oh well...
	    sizeToContent();
	}
	// update the airship times (I guess) - I kinda don't want to do this on every tick
	// because it isn't *really* necessary, but I can't really come up with any good
	// way to only do it when necessary since airship times fall on weird Vana'diel times
	for (var i in this.airshipTimes) {
	    this.airshipTimes[i] = (Math.floor(vanadielTime / airshipDelay) * airshipDelay) + airshipTimeDelays[i];
	    if (this.airshipTimes[i] < vanadielTime)
		this.airshipTimes[i] += airshipDelay;
	    this.airshipTimes[i] = VanadielClock_convertToEarthDate(this.airshipTimes[i]);
	}
    },

    // timer callback for Earth updates
    notify: function(timer) {
	// create a new date rounded to the nearest second
	var earthTime = new Date(Math.floor((new Date().getTime() + 500)/1000) * 1000);
	var earthTimeMillis = earthTime.getTime();
	var sec = earthTime.getSeconds();
	var min = earthTime.getMinutes();
	var hour;
    
	var label = formatEarthDate(earthTime) + " " +
		    formatEarthTime(earthTime);
	this.earthTimeLabel.setAttribute("value", label);
	// Update the new moon / full moon displays
	this.nextFullMoonLabel.setAttribute("value", formatEarthDelay(earthTime, this.nextFullMoonDate, true));
	this.nextNewMoonLabel.setAttribute("value", formatEarthDelay(earthTime, this.nextNewMoonDate, true));
	// Update the Selbina/Mhaura ferry timer
	this.nextFerryLabel.setAttribute("value", formatMilliDelay(this.earthNextFerry - earthTimeMillis));
	// Update airship delays.
	for (var i in this.airshipTimes) {
	    this.airshipElements[i].setAttribute("value", formatMilliDelay(this.airshipTimes[i].getTime() - earthTimeMillis));
	}
	// Update (only) the next RSE timer
	this.nextRSELabel.setAttribute("value", formatEarthDelay(earthTime, this.nextRSEDate, true));
	// Update guild delays, assuming the guilds have been built
	for (var i in guilds) {
	    label = guilds[i].isOpen ? guildCloseAt : guildOpenAt;
	    label += " ";
	    label += formatMilliDelay(guilds[i].changeAt.getTime() - earthTimeMillis);
	    this.guildTimers[i].setAttribute("value", label);
	}
	this.ballistaLabel.setAttribute("value", formatMilliDelay(this.nextBallista.getTime() - earthTime.getTime()));
	this.ballistaClosesLabel.setAttribute("value", formatMilliDelay(this.nextBallistaCloses.getTime() - earthTime.getTime()));
	if (this.timer != null) {
	    // try and resync back to the 1 second boundary
	    this.timer.initWithCallback(this, 1000 - (new Date().getTime() % 1000), Components.interfaces.nsITimer.TYPE_ONE_SHOT);
	}
    },
    
    // Update the ballista timers
    // Called every hour
    updateBallista: function() {
	// ballista runs over a six day cycle
	var day = this.clock.day;
	if (day == 30) // day 30 irks us
	    day = 0;
	var month = this.clock.month;
	var location = (day >> 1) % 3;
	var cap = Math.floor(day / 6);
	// the versus nations are choosen based on a four month repeating cycle
	var nation = Math.floor(month / 4);
	var elem = document.getElementById("ballista-nation-1");
	elem.setAttribute("nation", ballista_versus[nation][location][0]);
	elem.setAttribute("tooltiptext", ballista_strings[ballista_versus[nation][location][0]]);
	elem = document.getElementById("ballista-nation-2");
	elem.setAttribute("nation", ballista_versus[nation][location][1]);
	elem.setAttribute("tooltiptext", ballista_strings[ballista_versus[nation][location][1]]);
	elem = document.getElementById("ballista-location");
	elem.setAttribute("location", ballista_locations[location]);
	elem.setAttribute("tooltiptext", ballista_strings[ballista_locations[location]]);
	elem = document.getElementById("ballista-cap-label");
	elem.setAttribute("value", ballista_strings["level." + ballista_levels[cap]]);
	// Calculate the start of the entry period
	var vanadielTime = this.clock.time;
	var ballistaTime = this.clock.epochDays * 86400 + (((day ^ 1) & 1) * 86400) + 43200;
	var ballistaCloses = ballistaTime + 36000;
	// now go through and update the ballista rows for the upcoming events
	var earthTime = new Date();
	var date = VanadielClock_convertToEarthDate(ballistaTime);
	for (var i = 0; i < BALLISTA_ROWS; i++) {
	    day += 2;
	    date.setTime(date.getTime() + 6912000);
	    if (day > 29) {
		day -= 30;
		month++;
		if (month > 12)
		    month = 1;
	    }
	    // dump("cycling through " + day + "/" + month + "\n");
	    location = (day >> 1) % 3;
	    cap = Math.floor(day / 6);
	    nation = Math.floor(month / 4);
	    elem = document.getElementById("ballista-nation-1-" + i);
	    elem.setAttribute("nation", ballista_versus[nation][location][0]);
	    elem.setAttribute("tooltiptext", ballista_strings[ballista_versus[nation][location][0]]);
	    elem = document.getElementById("ballista-nation-2-" + i);
	    elem.setAttribute("nation", ballista_versus[nation][location][1]);
	    elem.setAttribute("tooltiptext", ballista_strings[ballista_versus[nation][location][1]]);
	    elem = document.getElementById("ballista-location-icon-" + i);
	    elem.setAttribute("location", ballista_locations[location]);
	    elem.setAttribute("tooltiptext", ballista_strings[ballista_locations[location]]);
	    elem = document.getElementById("ballista-location-label-" + i);
	    elem.setAttribute("value", ballista_strings[ballista_locations[location]]);
	    elem = document.getElementById("ballista-cap-label-" + i);
	    elem.setAttribute("value", ballista_strings["level." + ballista_levels[cap]]);
	    elem = document.getElementById("ballista-timer-label-" + i);
	    elem.setAttribute("value", formatEarthDelay(earthTime, date, false));
	}
	if (vanadielTime > ballistaTime && vanadielTime < ballistaCloses) {
	    // display the open entry labels
	    document.getElementById("ballista-entry-open").style.display = "-moz-box";
	    document.getElementById("ballista-entry-timer").style.display = "-moz-box";
	} else {
	    document.getElementById("ballista-entry-open").style.display = "none";
	    document.getElementById("ballista-entry-timer").style.display = "none";
	}
	elem = document.getElementById("ballisa-status-text");
	if (vanadielTime > ballistaTime) {
	    // move to the next period, guest entry
	    ballistaTime += 14400;
	    if (vanadielTime > ballistaTime) {
		// move to the next period, open entry
		ballistaTime += 7200;
		if (vanadielTime > ballistaTime) {
		    // move to the next period, the entry closing
		    ballistaTime += 14400;
		    if (vanadielTime > ballistaTime) {
			// move to the final period, the match starting
			ballistaTime += 7200;
			elem.setAttribute("value", ballistaMatch);
		    } else {
			elem.setAttribute("value", ballistaCloseEntry);
		    }
		} else {
		    elem.setAttribute("value", ballistaOpenEntry);
		}
	    } else {
		elem.setAttribute("value", ballistaGuestEntry);
	    }
	} else {
	    elem.setAttribute("value", ballistaEntry);
	}
	this.nextBallista = VanadielClock_convertToEarthDate(ballistaTime);
	this.nextBallistaCloses = VanadielClock_convertToEarthDate(ballistaCloses);
    },

    // Update the RSE timers.
    // Only called on new days.
    updateRSE: function() {
	// Create a list of RSE rows
	// Figure out which week this is...
	var vanadielWeek = Math.floor(this.clock.time / 691200);
	// Now start calculating the next 8 weeks (so we go through all races)
	var today = new Date();
	for (var i = 0; i < 8; i++,vanadielWeek++) {
	    var race = vanadielWeek % 8;
	    var location = vanadielWeek % 3;
	    var startTime = vanadielWeek * 691200;
	    var label;
	    if (i == 0) {
		label = RSEActive;
	    } else {
		label = formatEarthDelay(today, VanadielClock_convertToEarthDate(startTime), i == 1);
	    }
	    if (i == 1) {
		this.nextRSEDate = VanadielClock_convertToEarthDate(startTime);
	    }
	    document.getElementById("rse-icon-" + i).setAttribute("class", RSERaces[race].icon);
	    document.getElementById("rse-race-label-" + i).setAttribute("value", RSERaces[race].name);
	    document.getElementById("rse-location-label-" + i).setAttribute("value", RSELocations[location]);
	    document.getElementById("rse-time-label-" + i).setAttribute("value", label);
	}
    },

    // Update the guild status - called every Vana'diel hour
    updateGuilds: function(vanadielDay, vanadielHour) {
	for (var i in guilds) {
	    var guildRow = document.getElementById("guild-row-" + i);
	    var guildStatusLabel = document.getElementById("guild-status-label-" + i);
	    if ((vanadielDay % 8) == guilds[i].holiday) {
		// This guild is on holiday
		guildRow.setAttribute("status", "holiday");
		guildStatusLabel.setAttribute("value", guildHoliday);
		guilds[i].isOpen = false;
		// We change status tomorrow at open time
		guilds[i].changeAt = VanadielClock_convertToEarthDate(((vanadielDay + 1) * 86400) + (guilds[i].open * 3600));
	    } else {
		// Otherwise, see if we're open.
		if (guilds[i].open <= vanadielHour && guilds[i].close > vanadielHour) {
		    // We're open
		    guildRow.setAttribute("status", "open");
		    guildStatusLabel.setAttribute("value", guildOpen);
		    guilds[i].isOpen = true;
		    guilds[i].changeAt = VanadielClock_convertToEarthDate((vanadielDay * 86400) + (guilds[i].close * 3600));
		} else {
		    // We're closed
		    guildRow.setAttribute("status", "closed");
		    guildStatusLabel.setAttribute("value", guildClosed);
		    guilds[i].isOpen = false;
		    if (vanadielHour < guilds[i].open) {
			// Currently before our open time, we change at open today
			guilds[i].changeAt = VanadielClock_convertToEarthDate((vanadielDay * 86400) + (guilds[i].open * 3600));
		    } else {
			// Is tomorrow our holiday?
			if (((vanadielDay + 1) % 8) == guilds[i].holiday) {
			    // We change the day after
			    guilds[i].changeAt = VanadielClock_convertToEarthDate(((vanadielDay + 2) * 86400) + (guilds[i].open * 3600));
			} else {
			    // We change at open time tomorrow
			    guilds[i].changeAt = VanadielClock_convertToEarthDate(((vanadielDay + 1) * 86400) + (guilds[i].open * 3600));
			}
		    }
		}
	    }
	}
    }
}

function createRSERows() {
    var rows = document.getElementById("rse-rows");
    for (var i = 0; i < 8; i++) {
	var row = document.createElement("row");
	rows.appendChild(row);
	row.setAttribute("align", "center");
	var image = document.createElement("image");
	image.setAttribute("id", "rse-icon-" + i);
	row.appendChild(image);
	var label = document.createElement("label");
	label.setAttribute("id", "rse-race-label-" + i);
	row.appendChild(label);
	label = document.createElement("label");
	label.setAttribute("id", "rse-location-label-" + i);
	row.appendChild(label);
	label = document.createElement("label");
	label.setAttribute("id", "rse-time-label-" + i);
	row.appendChild(label);
    }
}

function createGuildRows() {
    var rows = document.getElementById("guild-rows");
    for (var i in guilds) {
	var row = document.createElement("row");
	rows.appendChild(row);
	row.setAttribute("align", "center");
	row.setAttribute("id", "guild-row-" + i);
	row.setAttribute("class", "guild-status");
	var image = document.createElement("image");
	image.setAttribute("id", "guild-icon-" + i);
	image.setAttribute("class", "guild-icon");
	image.setAttribute("guild", guilds[i].icon);
	row.appendChild(image);
	var label = document.createElement("label");
	label.setAttribute("id", "guild-name-label-" + i);
	label.setAttribute("value", guilds[i].name);
	row.appendChild(label);
	label = document.createElement("label");
	label.setAttribute("id", "guild-status-label-" + i);
	row.appendChild(label);
	label = document.createElement("label");
	label.setAttribute("id", "guild-timer-label-" + i);
	row.appendChild(label);
    }
}

function createBallistaRows(versus, at, longLabel) {
    var rows = document.getElementById("ballista-rows");
    for (var i = 0; i < BALLISTA_ROWS; i++) {
	var row = document.createElement("row");
	rows.appendChild(row);
	row.setAttribute("align", "center");
	row.setAttribute("id", "ballista-row-" + i);
	var image = document.createElement("image");
	image.setAttribute("id", "ballista-nation-1-" + i);
	image.setAttribute("class", "nation-icon");
	row.appendChild(image);
	var label = document.createElement("label");
	label.setAttribute("value", versus);
	row.appendChild(label);
	image = document.createElement("image");
	image.setAttribute("id", "ballista-nation-2-" + i);
	image.setAttribute("class", "nation-icon");
	row.appendChild(image);
	label = document.createElement("label");
	label.setAttribute("value", at);
	row.appendChild(label);
	image = document.createElement("image");
	image.setAttribute("id", "ballista-location-icon-" + i);
	image.setAttribute("class", "location-icon");
	row.appendChild(image);
	label = document.createElement("label");
	label.setAttribute("id", "ballista-location-label-" + i);
	//label.setAttribute("value", "");
	row.appendChild(label);
	label = document.createElement("label");
	label.setAttribute("id", "ballista-cap-label-" + i);
	row.appendChild(label);
	label = document.createElement("label");
	label.setAttribute("id", "ballista-timer-label-" + i);
	row.appendChild(label);
    }
}

function easterShowVersion() {
    var alertService = Components.classes["@mozilla.org/alerts-service;1"].getService(Components.interfaces.nsIAlertsService);
    alertService.showAlertNotification("chrome://vanadielclock/skin/icons/clock.png", "Vana'diel Clock", "Version 0.5.1", false, "don't care", null);
    var soundService = Components.classes["@mozilla.org/sound;1"].createInstance(Components.interfaces.nsISound);
    soundService.init();
    soundService.beep();
}

function updateTabPref() {
    Components.classes["@mozilla.org/preferences-service;1"].
               getService(Components.interfaces.nsIPrefBranch).
	       setIntPref("vanadiel.clock.tab", document.getElementById("detail-tabs").selectedIndex);
}
