/*
 * alarm-properties.js
 *
 * Copyright (c) 2005 by Daniel Potter.  All rights reserved.
 *
 * This file is part of the Vana'diel Clock.
 *
 * The Vana'diel Clock is free software; you can redistribute
 * it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * The Vana'diel Clock is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the Vana'diel Clock; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 *
 * Handles the UI for setting an alarm.
 */

const SOUND_FILES = "*.wav";

var rseRaces = ['elvaan.male', 'elvaan.female',
	'tarutaru.male', 'tarutaru.female',
	'mithra', 'galka', 'hume.male', 'hume.female' ];

var vanadielTimer;
var alarmManager;
var vanadielOffset = 0;
var edittedAlarm = null;

function init() {
    sizeToContent();
    // remove the width constraints on the description
    var desc = document.getElementById("advanced-description");
    desc.setAttribute("style", "width: " + desc.parentNode.innerWidth + "px");
    desc.removeAttribute("width");
    sizeToContent();
    var x = Math.floor((window.opener.outerWidth - window.outerWidth) / 2 + window.opener.screenX);
    var y = Math.floor((window.opener.outerHeight - window.outerHeight) / 2 + window.opener.screenY);
    window.moveTo(x, y);
    vanadielTimer = Components.classes["@xenoveritas.org/vanadiel-timer;1"]
			 .getService(Components.interfaces.nsIVanadielTimer);
    vanadielOffset = vanadielTimer.vanadielOffset;
    // grab the alarm manager
    alarmManager = Components.classes["@xenoveritas.org/vanadiel-alarm-manager;1"].
               getService(Components.interfaces.nsIVanadielAlarmManager);
    var label = document.getElementById("alarm.label");
    var aID = window.arguments.length >= 1 ? window.arguments[0] : null;
    if (aID == null || aID == "new") {
	document.getElementById("vanadielclock-set-alarm-window")
	    .setAttribute("title", document.getElementById("alarm-properties-strings").getString("window.title.create"));
    } else {
	var tabbox = document.getElementById("type-tabbox");
	// try and load an existing alarm from the alarm manager
	edittedAlarm = alarmManager.getAlarmById(aID);
	label.value = edittedAlarm.name;
	document.getElementById("alarm.description").value = edittedAlarm.description;
	if (edittedAlarm.sound == "none") {
	    document.getElementById("alarm-sound-group").selectedIndex = 0;
	} else if (edittedAlarm.sound == "beep") {
	    document.getElementById("alarm-sound-group").selectedIndex = 1;
	} else {
	    document.getElementById("alarm-sound-group").selectedIndex = 2;
	    document.getElementById("alarm-sound-custom-box").value = edittedAlarm.sound;
	}
	// Always default to "advanced" so something gets displayed even if
	// everything borks up
	tabbox.selectedTab = document.getElementById("advanced-tab");
	var type = edittedAlarm.type;
	var i = type.indexOf(" ");
	var pattern = edittedAlarm.pattern;
	if (i >= 0) {
	    pattern = type.substring(i + 1);
	    type = type.substring(0, i);
	}
	dump("Resurrecting alarm, [" + type + ":" + pattern + "]\n");
	// set up the existing data based on the alarm values
	if (type == "time") {
	    // figure out the day it sounds on
	    var toks = pattern.split(" ");
	    if (toks.length >= 3) {
		var day = parseInt(toks[0]);
		var menulist = document.getElementById("day-dropdown");
		if (day >= 0 && day <= menulist.menupopup.childNodes.length)
		    menulist.selectedIndex = day;
		tabbox.selectedTab = document.getElementById("time-tab");
		document.getElementById("time-box").value = toks[1] + ":" + ((parseInt(toks[2]) < 10) ? "0" : "") + toks[2];
	    }
	} else if (type == "moon") {
	    var menulist = document.getElementById("moon-dropdown");
	    // this is easy, find the matching pattern
	    tabbox.selectedTab = document.getElementById("moon-tab");
	    for (i = 0; i < menulist.menupopup.childNodes.length; i++) {
		if (menulist.menupopup.childNodes.item(i).getAttribute("pattern") == pattern) {
		    // select this
		    menulist.selectedIndex = i;
		    break;
		}
	    }
	} else if (type == "rse") {
	    var menulist = document.getElementById("rse-dropdown");
	    // this is easy, find the matching pattern
	    tabbox.selectedTab = document.getElementById("rse-tab");
	    for (i = 0; i < menulist.menupopup.childNodes.length; i++) {
		if (menulist.menupopup.childNodes.item(i).getAttribute("icon") == pattern) {
		    // select this
		    menulist.selectedIndex = i;
		    break;
		}
	    }
	} else if (type == "guild") {
	    var menulist = document.getElementById("guild-dropdown");
	    i = pattern.indexOf(" ");
	    var open = true;
	    if (i >= 0) {
		open = pattern.substring(i+1) != "close";
		pattern = pattern.substring(0, i);
	    }
	    document.getElementById("guild-radio-group").selectedIndex = open ? 0 : 1;
	    // this is easy, find the matching pattern
	    tabbox.selectedTab = document.getElementById("guild-tab");
	    for (i = 0; i < menulist.menupopup.childNodes.length; i++) {
		if (menulist.menupopup.childNodes.item(i).getAttribute("guild") == pattern) {
		    // select this
		    menulist.selectedIndex = i;
		    break;
		}
	    }
	}
	// synth the defaults, and if they match, kill 'em
	var alarm = createAlarm();
	if (label.value == alarm.name) {
	    label.value = "";
	}
	label = document.getElementById("alarm.description");
	if (label.value == alarm.description) {
	    label.value = "";
	}
	document.getElementById("advanced-textbox").value = alarm.pattern;
    }
    checkSoundBefore();
    checkAlarmSound();
    changedTab();
}

function changedTab() {
    var alarm = createAlarm();
    document.getElementById("alarm.label").defaultValue = alarm.name;
    document.getElementById("alarm.description").defaultValue = alarm.description;
}

function checkSoundBefore() {
    document.getElementById("minutes.before").disabled = ! document.getElementById("sound-before-event").checked;
}

function checkAlarmSound() {
    var disabled = !document.getElementById("alarm-sound-custom").selected;
    document.getElementById("alarm-sound-custom-box").disabled = disabled;
    document.getElementById("alarm-sound-custom-browse").disabled = disabled;
    // there's really no point to having "test" enabled if "none" is chosen
    // Actually, there's little point in having it enabled if "beep" is chosen,
    // but I suppose people might want to know what "system default" is
    document.getElementById("alarm-sound-custom-test").disabled = document.getElementById("alarm-sound-none").selected;
}

function browseSound() {
    var strings = document.getElementById("alarm-properties-strings");
    var nsIFilePicker = Components.interfaces.nsIFilePicker;
    var fp = Components.classes["@mozilla.org/filepicker;1"]
	.createInstance(nsIFilePicker);
    fp.appendFilter(strings.getString("sound.files") + " (" + SOUND_FILES + ")", SOUND_FILES);
    fp.init(window, strings.getString("sound.browse.title"), nsIFilePicker.modeOpen);
    var res = fp.show();
    if (res == nsIFilePicker.returnOK) {
	var soundFile = fp.file;
	document.getElementById("alarm-sound-custom-box").value = soundFile.path;
    }
}

function testSound() {
    // see what type of sound we're using
    if (document.getElementById("alarm-sound-none").selected) {
	// that was mighty pointless
	return;
    }
    var soundService = Components.classes["@mozilla.org/sound;1"].createInstance(Components.interfaces.nsISound);
    soundService.init();
    if (document.getElementById("alarm-sound-beep").selected) {
	soundService.beep();
    } else {
	soundService.playSystemSound(document.getElementById("alarm-sound-custom-box").value);
    }
}

/**
 * Dumb little interpolator.
 */
function quickyInterpolate(str, key1, val1, key2, val2) {
    var re = new RegExp("\\$\\{" + key1 + "\\}", "g");
    str = str.replace(re, val1);
    if (key2 != null) {
	re = new RegExp("\\$\\{" + key2 + "\\}", "g");
	str = str.replace(re, val2);
    }
    return str;
}

/**
 * Creates a skeleton object for a given alarm.  This is used to "test"
 * the alarm using the "Test Alarm" button, or at least the secret keystroke
 * Accel-T
 */
function createAlarm() {
    var alarm = new Object();
    alarm.name = "unknown";
    alarm.description = "unknown";
    alarm.sound = "none";
    if (document.getElementById("alarm-sound-beep").selected) {
	alarm.sound = "beep";
    } else if (document.getElementById("alarm-sound-custom").selected) {
	alarm.sound = document.getElementById("alarm-sound-custom-box").value;
    } else {
	alarm.sound = "none";
    }
    var vanadielTime = vanadielTimer.time;
    var type = document.getElementById("type-tabbox").selectedTab.getAttribute("id");
    var strings = document.getElementById("alarm-properties-strings");
    if (type == "time-tab") {
	// Create a new alarm based on the day.
	var item = document.getElementById("day-dropdown").selectedItem;
	var dayName = item.getAttribute("element");
	var day = -1;
	// convert that day back to a number
	for (var d in vanaDays) {
	    if (dayName == vanaDays[d]) {
		day = d;
		break;
	    }
	}
	if (day < 0) {
	    alert("Unknown day " + dayName);
	    return;
	}
	var hour = document.getElementById("time-box").value;
	var minute = hour.indexOf(":");
	if (minute >= 0) {
	    minute = parseInt(hour.substring(minute+1), 10);
	    if (!(minute >= 0 && minute < 60))
		minute = 0;
	} else {
	    minute = 0;
	}
	hour = parseInt(hour, 10);
	if (!(hour >= 0 && hour < 24))
	    hour = 0;
	alarm.smallIcon = "chrome://vanadielclock/skin/elements/" + dayName;
	alarm.icon = alarm.smallIcon + "32.png";
	alarm.smallIcon = alarm.smallIcon + ".png";
	alarm.name = quickyInterpolate(strings.getString("day.name"), "day", item.label);
	alarm.description = quickyInterpolate(strings.getString("day.desc"), "day", item.label, "time", hour + ":" + (minute < 10 ? "0" : "") + minute);
	alarm.pattern = day + " " + hour + " " + minute;
	alarm.type = "time " + alarm.pattern;
    } else if (type == "moon-tab") {
	// Create a new alarm based on the moon phase.
	var item = document.getElementById("moon-dropdown").selectedItem;
	alarm.smallIcon = "chrome://vanadielclock/skin/moon/" + item.getAttribute("icon") + ".png";
	alarm.icon = alarm.smallIcon;
	alarm.name = quickyInterpolate(strings.getString("moon.name"), "phase", item.label);
	alarm.description = quickyInterpolate(strings.getString("moon.desc"), "phase", item.label);
	alarm.pattern = item.getAttribute("pattern");
	alarm.type = "moon " + item.getAttribute("pattern");
    } else if (type == "rse-tab") {
	var item = document.getElementById("rse-dropdown").selectedItem;
	alarm.name = quickyInterpolate(strings.getString("rse.name"), "race", item.label);
	alarm.description = quickyInterpolate(strings.getString("rse.desc"), "race", item.label);
	alarm.pattern = item.getAttribute("pattern");
	alarm.type = "rse " + item.getAttribute("icon");
	alarm.smallIcon = "chrome://vanadielclock/skin/icons/races/" + item.getAttribute("icon");
	alarm.icon = alarm.smallIcon + "32.png";
	alarm.smallIcon = alarm.smallIcon + ".png";
    } else if (type == "guild-tab") {
	var item = document.getElementById("guild-dropdown").selectedItem;
	var open = document.getElementById("guild-radio-open").selected;
	var statusLocal = document.getElementById("guild-radio-" + (open ? "open" : "close")).label;
	alarm.name = quickyInterpolate(strings.getString("guild.name"), "guild", item.label, "status", statusLocal);
	alarm.description = quickyInterpolate(strings.getString("guild.desc"), "guild", item.label, "status", statusLocal);
	alarm.pattern = item.getAttribute("pattern-day") + " " + item.getAttribute("pattern-" + (open ? "open" : "close")) + " 0";
	alarm.type = "guild " + item.getAttribute("guild") + " " + (open ? "open" : "close");
    } else if (type == "advanced-tab") {
	alarm.name = strings.getString("advanced.name");
	alarm.description = strings.getString("advanced.desc");
	alarm.pattern = document.getElementById("advanced-textbox").value;
	alarm.type = "advanced";
    } else {
	//alert("You managed to choose an alarm type (" + type + ") that isn't listed.  Have a cookie!  But no alarm will be set, sorry.");
	//return;
    }
    return alarm;
}

function testAlarm() {
    var alarm = createAlarm();
    var alertService = Components.classes["@mozilla.org/alerts-service;1"].getService(Components.interfaces.nsIAlertsService);
    alertService.showAlertNotification(alarm.icon, alarm.name, alarm.description, false, "don't care", null);
    testSound();
}

function acceptChanges() {
    // Create the alarm
    var jsAlarm = createAlarm();
    var xpcAlarm = edittedAlarm == null ?
    			Components.classes["@xenoveritas.org/vanadiel-alarm;1"].
				createInstance(Components.interfaces.nsIVanadielAlarm) :
			edittedAlarm;
    // set the properties
    xpcAlarm.name = document.getElementById("alarm.label").value;
    xpcAlarm.description = document.getElementById("alarm.description").value;
    xpcAlarm.icon = jsAlarm.icon;
    xpcAlarm.smallIcon = jsAlarm.smallIcon;
    xpcAlarm.sound = jsAlarm.sound;
    xpcAlarm.type = jsAlarm.type;
    if (document.getElementById("sound-before-event").checked) {
	var i = parseInt(document.getElementById("minutes.before"));
	if (i > 0) {
	    i *= 60 * 25;
	} else {
	    i = 1 * 60 * 25;
	}
	dump("setting alarm to go off at " + i + " before\n");
	xpcAlarm.preAlertTime = i;
    }
    if (document.getElementById("alarm-repeat").checked) {
	xpcAlarm.pattern = jsAlarm.pattern;
	xpcAlarm.updateAlarm(VanadielClock_TimerService.time);
    } else {
	xpcAlarm.pattern = "";
	xpcAlarm.setAlarm(VanadielClock_TimerService.time, jsAlarm.pattern);
    }
    if (edittedAlarm == null)
	alarmManager.addAlarm(xpcAlarm);
    else if (window.opener.buildAlarmTree) {
	window.opener.buildAlarmTree();
    }
}
