/*
 * alarms.js
 *
 * Copyright (c) 2004 by Daniel Potter.  All rights reserved.
 *
 * This file is part of the Vana'diel Clock.
 *
 * The Vana'diel Clock is free software; you can redistribute
 * it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * The Vana'diel Clock is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the Vana'diel Clock; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 *
 * Handles the UI for setting alarms and the like.
 */

var editCommand;
var deleteCommand;
var alarmManager;
var ignoreManagerEvents = false;
var listener;

function VanadielClock_alarmLoad() {
    alarmManager = Components.classes["@xenoveritas.org/vanadiel-alarm-manager;1"].
               getService(Components.interfaces.nsIVanadielAlarmManager);
    localizeTimeFormatStrings();
    buildAlarmTree();
    listener = new Object();
    listener.alarmAdded = function(alarm) {
	if (!ignoreManagerEvents)
	    buildAlarmTree();
    };
    listener.alarmRemoved = function(alarm) {
	if (!ignoreManagerEvents)
	    buildAlarmTree();
    };
    alarmManager.addVanadielAlarmManagerListener(listener);
    editCommand = document.getElementById("cmdAlarmProperties");
    deleteCommand = document.getElementById("cmdAlarmDelete");
    // disable a couple of commands
    checkSelection();
}

function VanadielClock_alarmUnload() {
    if (listener != null) {
	alarmManager.removeVanadielAlarmManagerListener(listener);
	listener = null;
    }
}

function buildAlarmTree() {
    var tree = document.getElementById("alarm-tree");
    var children = document.getElementById("alarm-tree-children");
    // kill it and recreate it
    tree.removeChild(children);
    children = document.createElement("treechildren");
    children.setAttribute("id", "alarm-tree-children");
    children.setAttribute("flex", "1");
    tree.appendChild(children);
    var item, row, cell, time;
    for (var i = 0; i < alarmManager.count; i++) {
	var alarm = alarmManager.getAlarmAt(i);
	item = document.createElement("treeitem");
	item.setAttribute("id", "user-alarm-" + alarm.id);
	item.alarmObject = alarm;
	children.appendChild(item);
	row = document.createElement("treerow");
	item.appendChild(row);
	cell = document.createElement("treecell");
	cell.setAttribute("label", alarm.name);
	cell.setAttribute("src", alarm.smallIcon);
	row.appendChild(cell);
	cell = document.createElement("treecell");
	cell.setAttribute("label", alarm.description);
	row.appendChild(cell);
	cell = document.createElement("treecell");
	time = VanadielClock_convertToEarthDate(alarm.time);
	cell.setAttribute("label", formatEarthDate(time) + " " + formatEarthTime(time, false));
	row.appendChild(cell);
    }
}

function checkSelection() {
    var count = document.getElementById("alarm-tree").view.selection.count;
    if (count > 0) {
	if (count > 1) {
	    if (!editCommand.hasAttribute("disabled"))
		editCommand.setAttribute("disabled", "true");
	} else {
	    if (editCommand.hasAttribute("disabled"))
		editCommand.removeAttribute("disabled");
	}
	if (deleteCommand.hasAttribute("disabled"))
	    deleteCommand.removeAttribute("disabled");
    } else {
	editCommand.setAttribute("disabled", "true");
	deleteCommand.setAttribute("disabled", "true");
    }
}

function newAlarm() {
    VanadielClock_setAlarm();
}

function editAlarm() {
    var tree = document.getElementById("alarm-tree");
    var alarm = tree.view.getItemAtIndex(tree.view.selection.currentIndex).alarmObject;
    window.openDialog("chrome://vanadielclock/content/alarm-properties.xul", "_blank", "chrome", alarm.id);
}

function deleteAlarm() {
    var tree = document.getElementById("alarm-tree");
    var start = new Object();
    var end = new Object();
    var numRanges = tree.view.selection.getRangeCount();

    // since we're removeing (potentially) a bunch of objects at once,
    // ignore incoming events
    ignoreManagerEvents = true;
    for (var t = 0; t < numRanges; t++) {
	tree.view.selection.getRangeAt(t, start, end);
	for (var v = start.value; v <= end.value; v++) {
	    var alarm = tree.view.getItemAtIndex(v).alarmObject;
	    alarmManager.removeAlarm(alarm);
	}
    }
    ignoreManagerEvents = false;
    // kill the selection, otherwise the rebuild will muck stuff up
    tree.view.selection.clearSelection();
    buildAlarmTree();
}
