/*
 * options.js
 *
 * Copyright (c) 2005 by Daniel Potter.  All rights reserved.
 *
 * This file is part of the Vana'diel Clock.
 *
 * The Vana'diel Clock is free software; you can redistribute
 * it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * The Vana'diel Clock is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the Vana'diel Clock; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 *
 * Handles the setting and retrieving of options.
 */

var prefs = Components.classes["@mozilla.org/preferences-service;1"].
               getService(Components.interfaces.nsIPrefBranch);

var extPrefs;

var windowManager = Components.classes["@mozilla.org/appshell/window-mediator;1"].
		getService(Components.interfaces.nsIWindowMediator);

function init() {
    extPrefs = getVanadielClockPreferences(prefs);
    document.getElementById("showClock").checked = extPrefs.showClock;
    var moon = document.getElementById("showMoon");
    moon.checked = extPrefs.showMoon;
    // only can show the moon when the clock is active
    moon.disabled = ! extPrefs.showClock;
    var date = document.getElementById("showDate");
    // only can show the date when the clock is active
    date.checked = extPrefs.showDate;
    date.disabled = ! extPrefs.showClock;
    document.getElementById("use24HourClock").checked = extPrefs.use24HourClock;
    document.getElementById("minSeparator").setAttribute("value", extPrefs.minSeparator);
    document.getElementById("secSeparator").setAttribute("value", extPrefs.secSeparator);
    // Center this dialog on the opener
    var x = Math.floor((window.opener.outerWidth - window.outerWidth) / 2 + window.opener.screenX);
    var y = Math.floor((window.opener.outerHeight - window.outerHeight) / 2 + window.opener.screenY);
    window.moveTo(x, y);
}

function toggleClock() {
    var disabled = ! (document.getElementById("showClock").checked);
    document.getElementById("showMoon").disabled = disabled;
    document.getElementById("showDate").disabled = disabled;
}

function acceptChanges() {
    // called to commit current changes
    extPrefs.showClock = document.getElementById("showClock").checked;
    extPrefs.showMoon = document.getElementById("showMoon").checked;
    extPrefs.showDate = document.getElementById("showDate").checked;
    extPrefs.use24HourClock = document.getElementById("use24HourClock").checked;
    extPrefs.minSeparator = document.getElementById("minSeparator").value;
    extPrefs.secSeparator = document.getElementById("secSeparator").value;
    extPrefs.save(prefs);
    // go through windows and start/stop the clock appropriately
    // also if the SC window is up, apply pref changes to it, too
    var windowEnum = windowManager.getEnumerator(null);
    while (windowEnum.hasMoreElements()) {
	var win = windowEnum.getNext();
	if (win.updateRSE) {
	    win.earth24HourClock = extPrefs.use24HourClock;
	    win.minSeparator = extPrefs.minSeparator;
	    win.secSeparator = extPrefs.secSeparator;
	    win.updateRSE();
	}
	if (win.startVanadielClock) {
	    if (extPrefs.showClock) {
		win.startVanadielClock(extPrefs.showMoon, extPrefs.showDate);
	    } else {
		win.stopVanadielClock();
	    }
	}
    }
}
