/*
 * test.js
 *
 * Copyright (c) 2005 by Daniel Potter.  All rights reserved.
 *
 * This file is part of the Vana'diel Clock.
 *
 * The Vana'diel Clock is free software; you can redistribute
 * it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * The Vana'diel Clock is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the Vana'diel Clock; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 *
 * Runs several tests on the clock.
 */

var MINUTE = 60;
var HOUR = 60 * MINUTE;
var DAY = 24 * HOUR;
var WEEK = 8 * DAY;

var PATTERN_TESTS = [
	[ "0 0 0", 0, WEEK ],
	[ "1 0 0", 0, DAY ],
	[ "0,1,2,3,4 0 0", 0, DAY ],
	[ "0,1,2,3,4 0 0", DAY, DAY*2 ],
	[ "0,1,2,3,4 0 0", DAY*2, DAY*3 ],
	[ "0,1,2,3,4 0 0", DAY*2+1500, DAY*3 ],
	[ "0,1,2,3,4 0 0", DAY*5, DAY*8 ],
	[ "* 0 0 0", 0, (84-26) * DAY ],
	[ "* 5,21 0 38-44", 0, (38-26) * DAY + 5 * HOUR]
];

function runTests() {
    dump(" ==== STARTING TESTS ====\n");
    try {
	// start by creating a Vana'diel alarm.
	var alarm = Components.classes["@xenoveritas.org/vanadiel-alarm;1"].
		   createInstance(Components.interfaces.nsIVanadielAlarm);
	document.getElementById("pattern-test-image").setAttribute("test", runPatternTests(alarm));
	document.getElementById("alarm-test-image").setAttribute("test", runAlarmTests(alarm));
    } catch (ex) {
	dump("***** TESTS FAILED\n");
	dump("  An exception was thrown!\n");
	dump(ex);
	dump("\n");
	for (var key in ex) {
	    dump("exception.");
	    dump(key);
	    dump("=[");
	    dump(ex[key]);
	    dump("]\n");
	}
    }
}

function runPatternTests(alarm) {
    var v;
    var pass = 0;
    var fail = 0;
    var rows = document.getElementById("pattern-test-rows");
    var row;
    var elem;
    dump("Running pattern tests...\n");
    for (var t in PATTERN_TESTS) {
	row = document.createElement("row");
	rows.appendChild(row);
	dump(" - Running pattern test on pattern \"");
	dump(PATTERN_TESTS[t][0]);
	dump("\" starting on ");
	dump(PATTERN_TESTS[t][1]);
	dump(", expecting ");
	dump(PATTERN_TESTS[t][2]);
	alarm.setAlarm(PATTERN_TESTS[t][1], PATTERN_TESTS[t][0]);
	v = alarm.time;
	dump(", got ");
	dump(v);
	elem = document.createElement("image");
	elem.setAttribute("class", "test-image");
	if (v == PATTERN_TESTS[t][2]) {
	    dump(" - passed.\n");
	    pass++;
	    elem.setAttribute("test", "success");
	} else {
	    dump(" - FAILED.\n");
	    fail++;
	    elem.setAttribute("test", "fail");
	}
	row.appendChild(elem);
	elem = document.createElement("label");
	elem.setAttribute("value", PATTERN_TESTS[t][0] + ", wanted " + PATTERN_TESTS[t][2] + ", got " + v);
	row.appendChild(elem);
    }
    dump("Pattern tests complete, ");
    dump(pass);
    dump(" passed, ");
    dump(fail);
    dump(" failed.\n");
    if (fail == 0)
	return "success";
    if (pass == 0)
	return "fail";
    return "mixed";
}

function runAlarmTests(alarm) {
    var pass = 0;
    var fail = 0;
    dump("Running alarm tests...\n");
    alarm.preAlertTime = 1500;
    alarm.time = DAY;
    var image = document.getElementById("alarm-pre-test-image");
    if (alarm.nextAlarm == DAY - 1500) {
	pass++;
	image.setAttribute("test", "success");
    } else {
	fail++;
	image.setAttribute("test", "fail");
    }
    image = document.getElementById("alarm-at-test-image");
    alarm.activate(DAY - 1500);
    if (alarm.nextAlarm == DAY) {
	pass++;
	image.setAttribute("test", "success");
    } else {
	dump("Alarm at test failed, expected " + DAY + " but got " + alarm.nextAlarm + "\n");
	fail++;
	image.setAttribute("test", "fail");
    }
    image = document.getElementById("alarm-dead-test-image");
    alarm.activate(DAY);
    if (!alarm.active) {
	pass++;
	image.setAttribute("test", "success");
    } else {
	fail++;
	image.setAttribute("test", "fail");
    }
    dump("Alarm tests complete, ");
    dump(pass);
    dump(" passed, ");
    dump(fail);
    dump(" failed.\n");
    if (fail == 0)
	return "success";
    if (pass == 0)
	return "fail";
    return "mixed";
}
